<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 *
 * @author Sergey Kambalin <greyexpert@gmail.com>
 * @package ow_plugins.newsfeed.classes
 * @since 1.0
 */
class NEWSFEED_CLASS_EventHandler
{
    /**
     * 
     * @var NEWSFEED_BOL_Service
     */
    private $service;
    
    public function __construct()
    {
        $this->service = NEWSFEED_BOL_Service::getInstance();
    }
    
    private function validateParams( $params, $requiredList )
    {
        $fails = array();
        
        foreach ( $requiredList as $required )
        {
            if ( empty($params[$required]) )
            {
                $fails[] = $required;
            }
        }
        
        if ( !empty($fails) )
        {
            throw new InvalidArgumentException('Next params are required: ' . implode(', ', $fails));
        }
    }
    
    private function extractEventParams( OW_Event $event )
    {
        $defaultParams = array(
            'postOnUserFeed' => true,
            'privacy' => NEWSFEED_BOL_Service::PRIVACY_FULL,
            'replace' => false
        );
        
        $params = $event->getParams();
        $data = $event->getData();
        
        if ( empty($params['userId']) )
        {
            $params['userId'] = OW::getUser()->getId();
        }
        
        if ( isset($data['params']) && is_array($data['params']) )
        {
            $params = array_merge($params, $data['params']);    
        }
        
        return array_merge($defaultParams, $params);
    }
    
    private function extractEventData( OW_Event $event )
    {
        $data = $event->getData();
        unset($data['params']);
        
        return $data;
    }
    
    /**
     * 
     * @param OW_Event $event
     * @return NEWSFEED_CLASS_Action
     */
    private function convertEventToAction( OW_Event $event )
    {
        $params = $this->extractEventParams($event);
        $data = $this->extractEventData($event);
    
        $this->validateParams($params, array('entityType', 'entityId', 'userId', 'pluginKey'));
        
        if ( $params['replace'] || !isset($data['originalAction']) || $data['originalAction'] === null )
        {
            $action = new NEWSFEED_CLASS_Action();
            
            $action->setEntity($params['entityType'], $params['entityId']);
            $action->setPluginKey($params['pluginKey']);
            $action->setCreateTime(empty($data['time']) ? time() : $data['time']);
            $action->setUserId(empty($data['ownerId']) ? $params['userId'] : $data['ownerId']);
        }
        else
        {
            $action = $data['originalAction'];
        }
        
        if ( !empty($data['time']) )
        {
            $action->setCreateTime($data['time']);
        }
        
        $action->setPrivacy($params['privacy']);
        
        unset($data['originalAction']);
        unset($data['time']);
        unset($data['ownerId']);
        
        /* @var $action NEWSFEED_CLASS_Action */

        foreach ( $data as $name => $value )
        {
            $action->setDataValue($name, $value);
        }
        
        return $action;
    }
    
    public function action( OW_Event $originalEvent )
    {
        $params = $originalEvent->getParams();
        $this->validateParams($params, array('entityType', 'entityId'));

        $data = $originalEvent->getData();
        $data['originalAction'] = $this->service->getAction($params['entityType'], $params['entityId']);

        $event = new OW_Event('feed.on_entity_action', $params, $data);
        OW::getEventManager()->trigger($event);
        $params = $this->extractEventParams($event);
        $eventData = $this->extractEventData($event);
        
        $data = array_merge($data, empty($eventData) ? array() : $eventData);
        
        /* @var $action NEWSFEED_CLASS_Action */
        
        if ( $data['originalAction'] !== null )
        {
            $params['pluginKey'] = empty($params['pluginKey']) ? $data['originalAction']->getPluginKey() : $params['pluginKey'];  
            
            $event = new OW_Event('feed.on_entity_update', $params, $data);
            OW::getEventManager()->trigger($event);
            
            $params = $this->extractEventParams($event);

            if ( $params['replace'] )
            {
                $this->service->removeAction($data['originalAction']->getEntity()->type, $data['originalAction']->getEntity()->id);
            }
            
            $action = $this->convertEventToAction($event);
            $action->setUpdateTime(time());
        }
        else
        {
            $event = new OW_Event('feed.on_entity_add', $params, $data);
            OW::getEventManager()->trigger($event);
            
            $params = $this->extractEventParams($event);
            
            $addEventData = $event->getData();
            
            if ( empty($addEventData['content']) && empty($addEventData['string']) && empty($addEventData['line']) )
            {
                return;
            }
            
            $action = $this->convertEventToAction($event);
        }
        
        $this->service->saveAction($action);
        
        if ( $params['postOnUserFeed'] )
        {
            $this->service->addActionToFeed($action, 'user', (int) $params['userId']);            
        }
        
        if ( !empty( $params['feedType']) && !empty($params['feedId']) )
        {
            $this->service->addActionToFeed($action, $params['feedType'], (int) $params['feedId']);    
        }
    }
    
    public function addComment( OW_Event $e )
    {
        $this->action($e);
    }
    
    public function addLike( OW_Event $e )
    {
        $this->action($e);
    }
    
    public function addFriend( OW_Event $e )
    {
        $params = $e->getParams();
        $recipientId = $params['recipientId'];
        $senderId = $params['senderId'];
        
        $this->service->addFollow($recipientId, 'user', $senderId);
        $this->service->addFollow($senderId, 'user', $recipientId);
    }
    
    public function removeFriend( OW_Event $e )
    {
        $params = $e->getParams();
        $recipientId = $params['recipientId'];
        $senderId = $params['senderId'];
        
        $this->service->removeFollow($recipientId, 'user', $senderId);
        $this->service->removeFollow($senderId, 'user', $recipientId);
    }
    
    public function addFollow( OW_Event $e )
    {
        $params = $e->getParams();
        $this->validateParams($params, array('feedType', 'feedId', 'userId'));
        $this->service->addFollow($params['userId'], $params['feedType'], $params['feedId']);
    }
    
    public function removeFollow( OW_Event $e )
    {
        $params = $e->getParams();
        $this->validateParams($params, array('feedType', 'feedId', 'userId'));
        $this->service->removeFollow($params['userId'], $params['feedType'], $params['feedId']);
    }
    
    public function isFollow( OW_Event $e )
    {
        $params = $e->getParams();
        $this->validateParams($params, array('feedType', 'feedId', 'userId'));
        $result = $this->service->isFollow($params['userId'], $params['feedType'], $params['feedId']);
        $e->setData($result);
        
        return $result;
    }
    
    public function statusUpdate( OW_Event $e )
    {
        $params = $e->getParams();
        $data = $e->getData();

        $dto = NEWSFEED_BOL_Service::getInstance()->findStatusDto($params['feedType'], $params['feedId']);
        
        $eventParams = array(
            'pluginKey' => 'newsfeed',
            'entityType' => 'status',
            'entityId' => $dto->id,
            'postOnUserFeed' => false,
            'feedType' => $params['feedType'],
            'feedId' => $params['feedId']
        );
       
        if ( !empty($params['privacy']) )
        {
            $eventParams['privacy'] = (int) $params['privacy'];    
        }
        
        if ( !empty($data['privacy']) )
        {
            $eventParams['privacy'] = (int) $data['privacy'];    
        }
        
        OW::getEventManager()->trigger( new OW_Event('feed.action', $eventParams, array(
            'string' => $dto->status,
            'view' => array('iconClass' => 'ow_ic_comment')
        )));
    }
    
    public function installWidget( OW_Event $e )
    {
        $params = $e->getParams();
        
        $widgetService = BOL_ComponentAdminService::getInstance();
        
        try 
        {
            $widget = $widgetService->addWidget('NEWSFEED_CMP_EntityFeedWidget', false);
            $widgetPlace = $widgetService->addWidgetToPlace($widget, $params['place']);
            $widgetService->addWidgetToPosition($widgetPlace, $params['section'], $params['order']);
        }
        catch ( Exception $event )
        {
            $e->setData(false);
        }
        
        $e->setData($widgetPlace->uniqName);
    }
    
    public function deleteAction( OW_Event $e )
    {
        $params = $e->getParams();
        $this->validateParams($params, array('entityType', 'entityId'));
        
        $this->service->removeAction($params['entityType'], $params['entityId']);
    }
    
    public function onPluginDeactivate( OW_Event $e )
    {
        $params = $e->getParams();
        
        if ( $params['pluginKey'] == 'newsfeed' )
        {
            return;
        }
        
        $this->service->setActionStatusByPluginKey($params['pluginKey'], NEWSFEED_BOL_Service::ACTION_STATUS_INACTIVE);
    }
    
    public function onPluginActivate( OW_Event $e )
    {
        $params = $e->getParams();
        
        if ( $params['pluginKey'] == 'newsfeed' )
        {
            return;
        }
        
        $this->service->setActionStatusByPluginKey($params['pluginKey'], NEWSFEED_BOL_Service::ACTION_STATUS_ACTIVE);
    }
    
    public function onPluginUninstall( OW_Event $e )
    {
        $params = $e->getParams();
        
        if ( $params['pluginKey'] == 'newsfeed' )
        {
            return;
        }
        
        $this->service->removeActionListByPluginKey($params['pluginKey']);
    }
    
    public function getUserStatus( OW_Event $e )
    {
        $params = $e->getParams();
        
        $event = new OW_Event('feed.get_status', array(
            'feedType' => 'user',
            'feedId' => $params['userId'] 
        ));
        
        $this->getStatus($event);
        
        $e->setData($event->getData());
    }
    
    public function getStatus( OW_Event $e )
    {
        $params = $e->getParams();
        
        $status = $this->service->getStatus($params['feedType'], $params['feedId']);
        
        $e->setData($status);
    }
}