<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Photo upload action controller
 *
 * @author Egor Bulgakov <egor.bulgakov@gmail.com>
 * @package ow.plugin.photo.controllers
 * @since 1.0
 */
class PHOTO_CTRL_Upload extends OW_ActionController
{
    /**
     * @var PHOTO_BOL_PhotoService
     */
    private $photoService;
    /**
     * @var PHOTO_BOL_PhotoAlbumService
     */
    private $photoAlbumService;

    /**
     * Class constructor
     */
    public function __construct()
    {
        parent::__construct();

        $this->photoService = PHOTO_BOL_PhotoService::getInstance();
        $this->photoAlbumService = PHOTO_BOL_PhotoAlbumService::getInstance();

        if ( !OW::getRequest()->isAjax() )
        {
            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'photo', 'photo');
        }
    }

    /**
     * Default action
     */
    public function index()
    {
        // disallow not authenticated access
        if ( !OW::getUser()->isAuthenticated() )
        {
            throw new AuthenticateException();
        }

        $language = OW::getLanguage();
        $userId = OW::getUser()->getId();

        $config = OW::getConfig();
        $userQuota = (int) $config->getValue('photo', 'user_quota');

        if ( !OW::getUser()->isAuthorized('photo', 'upload') )
        {
            $this->assign('auth_msg', $language->text('photo', 'auth_upload_permissions'));
        }
        else if ( !($this->photoService->countUserPhotos($userId) <= $userQuota) )
        {
            $this->assign('auth_msg', $language->text('photo', 'quota_exceeded', array('limit' => $userQuota)));
        }
        else
        {
            $fileSizeLimit = $config->getValue('photo', 'accepted_filesize');
            $this->assign('limitMsg', $language->text('photo', 'size_limit', array('size' => $fileSizeLimit)));

            $this->assign('auth_msg', null);

            $photoUploadForm = new photoUploadForm();
            $this->addForm($photoUploadForm);

            $userId = OW::getUser()->getId();

            if ( OW::getRequest()->isPost() && $photoUploadForm->isValid($_POST) )
            {
                $res = $photoUploadForm->process();
                if ( $res['result'] )
                {
                    OW::getFeedback()->info($res['msg']);
                }
                else
                {
                    OW::getFeedback()->warning($res['msg']);
                }
                $this->redirect();
            }
        }

        OW::getDocument()->setHeading($language->text('photo', 'upload_photos'));
        OW::getDocument()->setHeadingIconClass('ow_ic_picture');

        $albumsUrl = OW::getRouter()->urlForRoute(
                'photo_user_albums',
                array('user' => BOL_UserService::getInstance()->getUserName($userId))
        );

        $js = new UTIL_JsGenerator();
        $js->newVariable('myAlbumsUrl', $albumsUrl);
        $js->jQueryEvent('#button-my-albums', 'click', 'window.location.href=myAlbumsUrl;');

        OW::getDocument()->addOnloadScript($js);

        $session = OW::getSession();

        if ( $session->isKeySet('photo.photoIds') )
        {
            $photoIds = $session->get('photo.photoIds');
            $recent = array();

            if ( $photoIds )
            {
                foreach ( $photoIds as $photoId )
                {
                    $recent[] = array(
                        'photoHref' => OW::getRouter()->urlForRoute('view_photo', array('id' => $photoId)),
                        'photoUrl' => PHOTO_BOL_PhotoService::getInstance()->getPhotoUrl($photoId, true)
                    );
                }
            }
            $this->assign('recent', $recent);

            $session->delete('photo.photoIds');
        }
    }

    /**
     * Prepare values for suggest field
     *
     * @param array $params
     */
    public function suggestAlbum( array $params )
    {
        $userId = trim($params['userId']);

        if ( OW::getRequest()->isAjax() )
        {
            $albums = $this->photoAlbumService->suggestUserAlbums($userId, $_GET['q']);

            if ( $albums )
            {
                foreach ( $albums as $album )
                {
                    echo "$album->name\t$album->id\n";
                }
            }
        }
        else
        {
            throw new Redirect404Exception();
            exit();
        }
    }
}

/**
 * Photo upload form class
 */
class photoUploadForm extends Form
{

    /**
     * Class constructor
     *
     */
    public function __construct()
    {
        parent::__construct('photoUploadForm');

        $language = OW::getLanguage();

        $this->setEnctype('multipart/form-data');

        // description Field
        $descField = new Textarea('description');
        $descField->setRequired(true);
        $this->addElement($descField->setLabel($language->text('photo', 'description')));

        $tagsField = new TagsField('tags');
        $this->addElement($tagsField->setLabel($language->text('photo', 'tags')));

        $filesNumber = 5;

        $labels = array();
        for ( $i = 0; $i < $filesNumber; $i++ )
        {
            $labels[$i] = $language->text('photo', 'pic_number', array('number' => $i + 1));
        }

        $filesField = new MultiFileField('photos', $filesNumber, $labels);
        $this->addElement($filesField);
        $filesField->setRequired(true);

        // album suggest Field
        $albumField = new SuggestField('album');
        $albumField->setRequired(true);
        $albumField->setMinChars(1);

        $userId = OW::getUser()->getId();
        $responderUrl = OW::getRouter()->urlFor('PHOTO_CTRL_Upload', 'suggestAlbum', array('userId' => $userId));

        $this->addElement(
                $albumField->setResponderUrl($responderUrl)
                ->setLabel($language->text('photo', 'album'))
        );

        $submit = new Submit('upload');
        $submit->setValue($language->text('photo', 'btn_upload'));
        $this->addElement($submit);
    }

    /**
     * Uploads photos
     *
     * @return boolean
     */
    public function process()
    {
        $values = $this->getValues();
        $photosArray = $values['photos'];

        $photoService = PHOTO_BOL_PhotoService::getInstance();
        $tagService = BOL_TagService::getInstance();
        $photoAlbumService = PHOTO_BOL_PhotoAlbumService::getInstance();

        $config = OW::getConfig();
        $configWidth = $config->getValue('photo', 'main_image_width');
        $configHeight = $config->getValue('photo', 'main_image_height');

        $configPreviewWidth = $config->getValue('photo', 'preview_image_width');
        $configPreviewHeight = $config->getValue('photo', 'preview_image_height');

        $uploadedCount = 0;
        $filledCount = 0;
        $newAlbum = false;

        if ( !strlen($albumName = htmlspecialchars(trim($_POST['album']))) )
        {
            return array('result' => false);
        }
        else
        {
            $userId = OW::getUser()->getId();

            // check album exists
            if ( !($album = $photoAlbumService->findAlbumByName($albumName, $userId)) )
            {
                $album = new PHOTO_BOL_PhotoAlbum();
                $album->name = $albumName;
                $album->userId = $userId;
                $album->createDatetime = time();

                $photoAlbumService->addAlbum($album);
                $newAlbum = true;
            }

            $filesCount = count($photosArray['name']);
            
            $photoIds = array();
            $cnfig = OW::getConfig();

            for ( $i = 0; $i < count($photosArray['name']); $i++ )
            {
                if ( strlen($photosArray['tmp_name'][$i]) )
                {
                    $filledCount++;
                    $accepted = floatval(OW::getConfig()->getValue('photo', 'accepted_filesize') * 1024 * 1024);

                    if ( !UTIL_File::validateImage($photosArray['name'][$i]) || $photosArray['size'][$i] > $accepted )
                        continue;

                    $photo = new PHOTO_BOL_Photo();
                    $photo->description = htmlspecialchars($values['description']);
                    $photo->albumId = $album->id;
                    $photo->addDatetime = time();
                    $photo->status = 'approved';
                    $photo->hasFullsize = false;

                    $photoService->addPhoto($photo);

                    if ( $photo->id )
                    {
                        BOL_TagService::getInstance()->updateEntityTags(
                            $photo->id,
                            'photo',
                            TagsField::getTags($values['tags'])
                        );

                        // destination path
                        $mainPhotoPath = $photoService->getPhotoPath($photo->id);
                        $previewPhotoPath = $photoService->getPhotoPath($photo->id, 'preview');
                        $originalPhotoPath = $photoService->getPhotoPath($photo->id, 'original');

                        // pluginfiles tmp path
                        $mainPhotoPFPath = $photoService->getPhotoPluginFilesPath($photo->id);
                        $previewPhotoPFPath = $photoService->getPhotoPluginFilesPath($photo->id, 'preview');

                        $storage = OW::getStorage();

                        if ( !is_writable(dirname($mainPhotoPFPath)) )
                        {
                            $photoService->deletePhoto($photo->id);
                            continue;
                        }

                        try
                        {
                            $image = new UTIL_Image($photosArray['tmp_name'][$i]);

                            $resized = $image
                                    ->resizeImage($configWidth, null)
                                    ->saveImage($mainPhotoPFPath);

                            $storage->copyFile($mainPhotoPFPath, $mainPhotoPath);
                            @unlink($mainPhotoPFPath);

                            $photo->hasFullsize = (bool) $config->getValue('photo', 'store_fullsize') && $resized->imageResized();
                            $photoService->updatePhoto($photo);

                            if ( $photo->hasFullsize )
                            {
                                $storage->copyFile($photosArray['tmp_name'][$i], $originalPhotoPath);
                            }

                            $resized
                                ->resizeImage($configPreviewWidth, $configPreviewHeight, true)
                                ->saveImage($previewPhotoPFPath);

                            $storage->copyFile($previewPhotoPFPath, $previewPhotoPath);
                            @unlink($previewPhotoPFPath);

                            $uploadedCount++;
                            array_push($photoIds, $photo->id);
                        }
                        catch ( WideImage_Exception $e )
                        {
                            $photoService->deletePhoto($photo->id);

                            OW::getSession()->set('photo.photoIds', $photoIds);
                            $msg = OW::getLanguage()->text('photo', 'photo_upload_error');
                            return array('result' => false, 'msg' => $msg);
                        }
                    }
                }
            }

            if ( !$uploadedCount && $newAlbum )
            {
                $photoAlbumService->deleteAlbum($album->id);
            }

            if ( $uploadedCount )
            {
                if ( $uploadedCount == 1 )
                {
                    $url = OW::getRouter()->urlForRoute('view_photo', array('id' => $photo->id));

                    $data = array(
                        'string' => OW::getLanguage()->text('photo', 'activity_string',
                            array(
                                'user' => BOL_UserService::getInstance()->getDisplayName($userId),
                                'userUrl' => BOL_UserService::getInstance()->getUserUrl($userId),
                                'url' => $url
                            )
                        ),
                        'content' => '<a href="' . $url . '"><img src="' . $photoService->getPhotoUrl($photo->id, true) . '" height="80" /></a>'
                    );

                    //Newsfeed
                    $event = new OW_Event('feed.action', array(
                        'pluginKey' => 'photo',
                        'entityType' => 'photo_comments',
                        'entityId' => $photo->id,
                        'userId' => $userId
                    ));
                    OW::getEventManager()->trigger($event);
                    
                    $entityId = $photo->id;
                }
                else
                {
                    $content = '';

                    foreach ( $photoIds as $id )
                    {
                        $pageUrl = $url = OW::getRouter()->urlForRoute('view_photo', array('id' => $id));
                        $content .= '<a href="' . $pageUrl . '"><img src="' . $photoService->getPhotoUrl($id, true) . '" height="80" /></a> ';
                    }

                    $data = array(
                        'string' => OW::getLanguage()->text('photo', 'activity_string_multiple',
                            array(
                                'user' => BOL_UserService::getInstance()->getDisplayName($userId),
                                'userUrl' => BOL_UserService::getInstance()->getUserUrl($userId),
                                'albumUrl' => OW::getRouter()->urlForRoute('photo_user_album', array('user' => $userId, 'album' => $photo->albumId)),
                                'albumName' => $album->name,
                                'count' => $uploadedCount
                            )
                        ),
                        'content' => $content
                    );

                    $entityId = $photo->albumId;
                    
                    //Newsfeed
                    $albumUrl = OW::getRouter()->urlForRoute('photo_user_album', array(
                        'user' => BOL_UserService::getInstance()->getUserName($album->userId),
                        'album' => $album->id
                    ));
                    $albumName = UTIL_String::truncate(strip_tags($album->name), 50, '...');
                    $title = UTIL_String::truncate(strip_tags($photo->description), 100, '...');
                    
                    $event = new OW_Event('feed.action', array(
                        'pluginKey' => 'photo',
                        'entityType' => 'multiple_photo_upload',
                        'entityId' => $photoIds[0],
                        'userId' => $userId
                    ), array(
                        'string' => $title,
                        'content' => $data['content'],
                        'view' => array( 'iconClass' => 'ow_ic_picture' ),
                        'toolbar' => array(array(
                            'href' => $albumUrl,
                            'label' => OW::getLanguage()->text('photo', 'feed_goto_album_label', array('album' => $albumName))
                        ))
                    ));
                    OW::getEventManager()->trigger($event);
                }

                if ( OW::getPluginManager()->isPluginActive('activity') )
                {
                    $action = new ACTIVITY_BOL_Action();
                    
                    $action->setUserId($userId)
                        ->setTimestamp(time())
                        ->setType('photo-upload')
                        ->setEntityId($entityId)
                        ->setData($data);
    
                    ACTIVITY_BOL_Service::getInstance()->addAction($action);
                }
            }

            OW::getSession()->set('photo.photoIds', $photoIds);

            if ( $uploadedCount == 0 )
            {
                $msg = OW::getLanguage()->text('photo', 'no_photo_uploaded');
                $return = array('result' => false, 'msg' => $msg);
            }
            else if ( $uploadedCount < $filledCount )
            {
                $msg = OW::getLanguage()->text('photo', 'not_all_photos_uploaded');
                $return = array('result' => false, 'msg' => $msg);
            }
            else
            {
                $msg = OW::getLanguage()->text('photo', 'photos_uploaded', array('count' => $uploadedCount));
                $return = array('result' => true, 'msg' => $msg);
            }
            return $return;
        }
    }
}